/**
  * File: DomainGroupHandlerBaseImpl.java
  * Content: Defines an abstract RMI class to implement the DomainGroupHandler interface
  * Author: LuisM Pena
  * Date: 29th November 2000
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.middleware.domains;

import org.omg.CORBA.UserException;

import sensei.middleware.gms.ORBcentral;

/**
  * Implementation of the GroupMember CORBA interface.
  **/
public abstract class DomainGroupHandlerBaseImpl extends DomainGroupHandlerPOA {

	public DomainGroupHandlerBaseImpl() throws UserException {
		this(true);
	}

	public DomainGroupHandlerBaseImpl(boolean active) throws UserException {
		if (active)
			activate();
	}

	public void activate() throws UserException {
		if (thisDomainGroupHandler == null) {
			id = ORBcentral.getPOA().activate_object(this);
			thisDomainGroupHandler = DomainGroupHandlerHelper.narrow(ORBcentral.getPOA().id_to_reference(id));
		}
	}

	public void deactivate() throws UserException {
		synchronized (this) {
			if (thisDomainGroupHandler != null)
				ORBcentral.getPOA().deactivate_object(id);
			thisDomainGroupHandler = null;
		}
	}

	public final DomainGroupHandler theDomainGroupHandler() {
		return thisDomainGroupHandler;
	}

	protected DomainGroupHandler thisDomainGroupHandler;
	byte[] id;
};
