/**
  * File: MemberInfo.java
  * Content: Helper class containing state information on each member on the group
  * Author: LuisM Pena
  * Date: 16th April 2001
  * Version: 0.51.00
  * Last change:
  *
  *
  **/

package sensei.domains;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
  * Helper class containing state information on each member on the group
  **/
class MemberInfo {
	public MemberInfo() {
		Trace.code("MemberInfo.java -> MemberInfo ( )");
		coordinating = new ArrayList();
		returnCoordinating = null;
	}

	public synchronized void freeResources() {
		Trace.code("MemberInfo.java -> synchronized void freeResources ( )");
		if (coordinating != null) {
			coordinating.clear();
			coordinating = null;
			returnCoordinating = null;
		}
	}

	public synchronized void addMemberToCoordinate(Integer member) {
		Trace.code("MemberInfo.java -> synchronized void addMemberToCoordinate ( Integer member )");
		coordinating.add(member);
		returnCoordinating = null;
	}

	public synchronized boolean removeMemberToCoordinate(Integer member) {
		Trace.code("MemberInfo.java -> synchronized boolean removeMemberToCoordinate ( Integer member )");
		returnCoordinating = null;
		return coordinating.remove(member);
	}

	public synchronized Integer getMemberToCoordinate() {
		Trace.code("MemberInfo.java -> synchronized Integer getMemberToCoordinate ( )");
		return coordinating.isEmpty() ? null : (Integer) (coordinating.get(0));
	}

	/**
	 * Adds to the target the list of members being coordinated
	 */
	public synchronized void getCoordinatedMembers(Collection target) {
		Trace.code("MemberInfo.java -> synchronized void getCoordinatedMembers ( Collection target )");
		target.addAll(coordinating);
	}

	public synchronized int[] getCoordinatedMembers() {
		Trace.code("MemberInfo.java -> synchronized int [ ] getCoordinatedMembers ( )");
		if (returnCoordinating == null)
			returnCoordinating = getTheCoordinatedMembers();
		return returnCoordinating;
	}

	int[] getTheCoordinatedMembers() {
		Trace.code("MemberInfo.java -> int [ ] getTheCoordinatedMembers ( )");
		int howmany = coordinating.size();
		int ret[] = new int[howmany];
		if (howmany > 0) {
			int index = 0;
			Iterator iterator = coordinating.iterator();
			while (iterator.hasNext()) {
				Integer member = (Integer) (iterator.next());
				ret[index++] = member.intValue();
			}
			assert index == howmany;
		}
		return ret;
	}

	List coordinating; //members being coordinated. Array of GroupMemberIds
	int[] returnCoordinating;
}
