/**
  * File: TokenPassControl.java
  * Content: The token should not be passing from member to member if there is no
  *					activity in the group. This class retains the token if there is no such
  *					activity
  * Author: LuisM Pena
  * Date: 5th September 2000
  * Version: 0.97.00
  * Last change:
  * Comments: The current algorithm is rather simple, as it only controls the
  *						activity during the last round and the time retaining the token
  *           doesn't depend on the number of group members
  **/

package sensei.gms;

import sensei.util.SenseiError;

/**
  * The token should not be passing from member to member if there is no
  *	activity in the group. This class retains the token if there is no such
  *	activity<p>
  * The current algorithm is rather simple, as it only controls the
  *	activity during the last round and the time retaining the token
  * doesn't depend on the number of group members
  **/
class TokenPassControl {

	//*************************************************************************************//
	//**************************** GROUP COMMUNICATION ************************************//
	//*************************************************************************************//

	/**
	  * Informs to this class that the member has received a valid group communication
	  **/
	public void groupCommunication() {
		Trace.code("TokenPassControl.java -> void groupCommunication ( )");
		activity = true;
	}

	//*************************************************************************************//
	//**************************** TO PASS TOKEN ******************************************//
	//*************************************************************************************//

	/**
	  * Informs to this class that the member is going to send a token.
	  * It will be checked the activity happened on the last round and this call will be
	  * retained if the algorithm considers it.
	  **/
	public void toPassToken() {
		Trace.code("TokenPassControl.java -> void toPassToken ( )");
		if (!activity) {
			try {
				Thread.sleep(RETAIN_TOKEN_PERIOD);
			}
			catch (InterruptedException ex) {
				SenseiError.unhandledException(Consts.GMS, ex);
			}
		}
		activity = false;
	}

	//*************************************************************************************//
	//**************************** DATA MEMBERS *******************************************//
	//*************************************************************************************//

	boolean activity = true;

	static final long RETAIN_TOKEN_PERIOD = Consts.RETAIN_TOKEN_PERIOD;
}
