/**
  * File: WrongActiveGroupMember.java
  * Content: ActiveGroupMember exhibiting a wrong behaviour
  * Author: LuisM Pena
  * Date: 16nd July 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.InternalViewId;
import sensei.middleware.gms.SenseiGMSMember;
import sensei.middleware.gms.Token;
import sensei.util.RestartableThread;

/**
  * ActiveGroupMember exhibiting a wrong behaviour
  **/
public class WrongActiveGroupMember extends ActiveGroupMember {
	public WrongActiveGroupMember(GroupMember observer, WrongBehaviour behaviour) throws Exception {
		super(observer);
		Trace.code("WrongActiveGroupMember.java -> WrongActiveGroupMember ( GroupMember observer , WrongBehaviour behaviour )");
		this.behaviour = behaviour;
		delayJoin = behaviour.delayJoinOfMembersUntilTemporalView();
	}

	//*************************************************************************************//
	//**************************** SEND VIEW IF NEEDED ************************************//
	//*************************************************************************************//

	boolean sendViewIfNeeded(boolean sendViewAnyway) {
		Trace.code("WrongActiveGroupMember.java -> boolean sendViewIfNeeded ( boolean sendViewAnyway )");
		boolean sentOk = true;
		boolean sendNeeded = sendViewAnyway;

		//--> FROM HERE IT CHANGES
		SenseiGMSMember[] newMembers = null;
		if (!(delayJoin && (viewStatus == VIEW_INSTALLED)))
			newMembers = joiningMembers.getMembers();
		//<-- CHANGES UP TO HERE

		//it's needed to send a new view if there are new members or faulty ones
		if (newMembers != null) {
			viewHandler.addNewChannels(newMembers);
			sendNeeded = true;
		}
		else
			sendNeeded = viewHandler.faultyMembersToCommunicate();

		if (sendNeeded || sendViewAnyway)
			sentOk = sendView();

		if (newMembers != null)
			//this call wakes up the members waiting to be joined
			joiningMembers.removeMembers(newMembers);

		return sentOk;
	}

	//*************************************************************************************//
	//**************************** START GROUP ACTIVITES **********************************//
	//*************************************************************************************//

	boolean startGroupActivities(InternalView initialView, Token initialToken) {
		Trace.code("WrongActiveGroupMember.java -> boolean startGroupActivities ( InternalView initialView , Token initialToken )");
		boolean started = false;
		synchronized (this) {
			started = !leaving && (viewStatus == MEMBER_NOT_IN_VIEW);
			if (started) {
				view = new InternalView(new InternalViewId(initialView.viewId.id, initialView.viewId.installing), initialView.members, initialView.memberIds, initialView.newMembers);
				memberRecoveringToken = Consts.INVALID_MEMBER_ID;
				viewStatus = RECEIVED_TEMPORAL_VIEW;
				memberAccepted = false;

				restartableThread = new RestartableThread(this);
				/**/
				viewHandler = new WrongViewHandler(thisSenseiGMSMember, initialView, behaviour);
				thisMemberId = viewHandler.getMyId();
				tokenState = new TokenState(thisMemberId, viewHandler, initialToken);
				eventsDispatcher = new GroupEventsDispatcher(thisMemberId, thisSenseiGMSMember, groupMember);
				/**/
				messageCenter = new WrongMessageCenter(eventsDispatcher, behaviour);
				joiningMembers = new JoiningMembersHandler();
				tokenRecoverer = new TokenRecoverer(this, thisSenseiGMSMember, initialView);
				/**/
				if (behaviour.passTokenSlowly())
					/**/
					tokenControl = new WrongTokenPassControl();
				/**/
				else
					tokenControl = new TokenPassControl();

				Trace.viewReceived(initialView);
				Trace.viewAccepted(initialView, thisMemberId);
			}
		}
		return started;
	}
	boolean delayJoin;
	WrongBehaviour behaviour;
}
