/**
  * File: WrongBehaviour.java
  * Content: definition of the wrong beahviour
  * Author: LuisM Pena
  * Date: 12th August 2000
  * Version: 0.97.00
  * Last change:
  *
  * Comments: useful error codes:
  *						128: messages sent on flush
  *						256: messages sent on next view
  *						512: members joining are delayed to next view
  *					 1024: token is passed slowly
  **/

package sensei.gms;

public class WrongBehaviour {
	public static int MESSAGES_WRONG = 0;
	public static int MESSAGES_SENT_TO_ALL_BUT_ONE = 1;
	public static int MESSAGES_EXIT_AFTER_SENT = 2;

	public static int VIEW_WRONG = 3;
	public static int VIEW_SENT_TO_ALL_BUT_ONE = 4;
	public static int VIEW_EXIT_AFTER_SENT = 5;

	public static int MESSAGE_CONFIRMATION_WRONG = 6;

	public static int DELAY_SEND_OF_MESSAGES_TO_FLUSH = 7;
	public static int DELAY_SEND_OF_MESSAGES_TO_NEXT_VIEW = 8;

	public static int DELAY_JOIN_UNTIL_TEMPORAL_VIEW = 9;
	public static int PASS_TOKEN_SLOWLY = 10;

	public WrongBehaviour(int behaviour) {
		Trace.code("WrongBehaviour.java -> WrongBehaviour ( int behaviour )");
		this.behaviour = behaviour;
	}

	public boolean isWrong() {
		Trace.code("WrongBehaviour.java -> boolean isWrong ( )");
		return behaviour != 0;
	}

	public boolean isMessagesWrong() {
		Trace.code("WrongBehaviour.java -> boolean isMessagesWrong ( )");
		return ((behaviour >> MESSAGES_WRONG) & 1) != 0;
	}

	public boolean isMessagesToSentToAllButOne() {
		Trace.code("WrongBehaviour.java -> boolean isMessagesToSentToAllButOne ( )");
		return ((behaviour >> MESSAGES_SENT_TO_ALL_BUT_ONE) & 1) != 0;
	}

	public boolean exitAfterMessagesSent() {
		Trace.code("WrongBehaviour.java -> boolean exitAfterMessagesSent ( )");
		return ((behaviour >> MESSAGES_EXIT_AFTER_SENT) & 1) != 0;
	}

	public boolean isMessageConfirmationWrong() {
		Trace.code("WrongBehaviour.java -> boolean isMessageConfirmationWrong ( )");
		return ((behaviour >> MESSAGE_CONFIRMATION_WRONG) & 1) != 0;
	}

	public boolean isViewWrong() {
		Trace.code("WrongBehaviour.java -> boolean isViewWrong ( )");
		return ((behaviour >> VIEW_WRONG) & 1) != 0;
	}

	public boolean isViewToSentToAllButOne() {
		Trace.code("WrongBehaviour.java -> boolean isViewToSentToAllButOne ( )");
		return ((behaviour >> VIEW_SENT_TO_ALL_BUT_ONE) & 1) != 0;
	}

	public boolean exitAfterViewSent() {
		Trace.code("WrongBehaviour.java -> boolean exitAfterViewSent ( )");
		return ((behaviour >> VIEW_EXIT_AFTER_SENT) & 1) != 0;
	}

	public boolean delaySendOfMessages() {
		Trace.code("WrongBehaviour.java -> boolean delaySendOfMessages ( )");
		return delaySendOfMessagesToNextView() || delaySendOfMessagesToFlush();
	}

	public boolean delaySendOfMessagesToNextView() {
		Trace.code("WrongBehaviour.java -> boolean delaySendOfMessagesToNextView ( )");
		return ((behaviour >> DELAY_SEND_OF_MESSAGES_TO_NEXT_VIEW) & 1) != 0;
	}

	public boolean delaySendOfMessagesToFlush() {
		Trace.code("WrongBehaviour.java -> boolean delaySendOfMessagesToFlush ( )");
		return ((behaviour >> DELAY_SEND_OF_MESSAGES_TO_FLUSH) & 1) != 0;
	}

	public boolean delayJoinOfMembersUntilTemporalView() {
		Trace.code("WrongBehaviour.java -> boolean delayJoinOfMembersUntilTemporalView ( )");
		return ((behaviour >> DELAY_JOIN_UNTIL_TEMPORAL_VIEW) & 1) != 0;
	}

	public boolean passTokenSlowly() {
		Trace.code("WrongBehaviour.java -> boolean passTokenSlowly ( )");
		return ((behaviour >> PASS_TOKEN_SLOWLY) & 1) != 0;
	}

	int behaviour;

}
