/**
  * File: WrongCastMessagesChannelRequester.java
  * Content: Defines a channel requester to cast a message who behaves faulty.
  *					The faulty behaviour means that the message won't be sent to every
  *					member (it can be sent to one or to all but one)
  * Author: LuisM Pena
  * Date: 12th August 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.Message;
import sensei.middleware.gms.MessageId;

//*************************************************************************************//
//**************************** WRONG CAST MESSAGE CHANNEL REQUESTER *******************//
//*************************************************************************************//

/**
  * Channel Requester used to cast messages but with a faulty behaviour:
  * The faulty behaviour means that the message won't be sent to every
  *					member (it can be sent to one or to all but one)
  **/
class WrongCastMessagesChannelRequester extends CastMessagesChannelRequester {

	/**
	  * @param sentToAllButOne if this parameter is set to true, the messages are
		  *      	sent to every member but the first one; otherwise, it's only sent to the
		  *      	first one. (The owner will also receive it)
	  **/
	public WrongCastMessagesChannelRequester(int owner, boolean sendToAllButOne) {
		super(owner);
		Trace.code("WrongCastMessagesChannelRequester.java -> WrongCastMessagesChannelRequester ( int owner , boolean sendToAllButOne )");
		this.sendToAllButOne = sendToAllButOne;
		invocation = 0;
	}

	public void setMessagesToSend(Message[] messages, MessageId messageId) {
		Trace.code("WrongCastMessagesChannelRequester.java -> void setMessagesToSend ( Message [ ] messages , MessageId messageId )");
		super.setMessagesToSend(messages, messageId);
		invocation = 0;
	}

	public boolean invokeMember(CommunicationChannel channel) {
		Trace.code("WrongCastMessagesChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
		assert channel != null;
		boolean invokeSuper = false;
		;
		if (channel.getLinkId() == thisMember)
			invokeSuper = true;
		else if (invocation++ == 0)
			invokeSuper = !sendToAllButOne;
		else
			invokeSuper = sendToAllButOne;
		return invokeSuper ? super.invokeMember(channel) : false;
	}

	int invocation;
	boolean sendToAllButOne;
}
