/**
  * File: WrongTokenPassControl.java
  * Content: The token should not be passing from member to member if there is no
  *					activity in the group. This class retains the token if there is no such
  *					activity. The wrong behaviour means that the slow down of the token
  *					happens always.
  * Author: LuisM Pena
  * Date: 3rd November 2000
  * Version: 0.97.00
  * Last change:
  **/

package sensei.gms;

import sensei.util.SenseiError;

class WrongTokenPassControl extends TokenPassControl {

	//*************************************************************************************//
	//**************************** TO PASS TOKEN ******************************************//
	//*************************************************************************************//

	public void toPassToken() {
		Trace.code("WrongTokenPassControl.java -> void toPassToken ( )");
		try {
			Thread.sleep(SLOW_TOKEN_PERIOD);
		}
		catch (InterruptedException ex) {
			SenseiError.unhandledException(Consts.GMS, ex);
		}
	}

	//*************************************************************************************//
	//**************************** DATA MEMBERS *******************************************//
	//*************************************************************************************//

	//slowing down the period that is considered the token to be lost
	static final long SLOW_TOKEN_PERIOD = Consts.RECOVER_TOKEN_PERIOD / 2;
}
