/**
  * File: SenseiGMSMemberImpl.java
  * Content: Implementation of the GroupMember RMI interface. This implementation is intended
  * 				to allow to the application to define its GroupMember class directly inheriting
  *					from this class.
  * Author: LuisM Pena
  * Date: 17th November 2000
  * Version: 0.80.00
  * Last change:
  *
  **/

package sensei.middleware.gms;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
  * Class implementing the RMI interface for the GroupMember. This implementation is intended
  * to allow to the application to define its GroupMember class directly inheriting
  *	from this class.
  **/
public abstract class SenseiGMSMemberImpl extends UnicastRemoteObject implements SenseiGMSMember {

	public SenseiGMSMemberImpl() throws RemoteException {
		thisSenseiGMSMember = this;
	}

	public SenseiGMSMemberImpl(boolean activate) throws RemoteException {
		if (!activate)
			deactivate();
	}

	public void activate() throws RemoteException {
		if (thisSenseiGMSMember == null) {
			thisSenseiGMSMember = this;
			exportObject(this);
		}
	}

	public void deactivate() throws RemoteException {
		synchronized (this) {
			if (thisSenseiGMSMember != null)
				unexportObject(thisSenseiGMSMember, true);
			thisSenseiGMSMember = null;
		}
	}

	public final SenseiGMSMember theSenseiGMSMember() {
		return thisSenseiGMSMember;
	}

	SenseiGMSMember thisSenseiGMSMember;
}
