/**
  * File: Rerunnable.java
  * Content: Interface to be implemented by classes using the RestartableThread
  *          mechanisms
  * Author: LuisM Pena
  * Date: 17th November 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.util;

/**
  * A thread can not be started more than once, and when it is activated the
  * method run() is called. Once this methos finish, the thread is ended and
  * cannot be used again.
  * This interface, together with the RestartableThread class, overpass that
  * limitation, allowing the restarting of the thread.
  **/
public interface Rerunnable {
	/**
	  * Called when the thread is finally activated.
	  * @param taskParam the argument passed to activate in the RestartableThread
	  * @returns true if the thread must continue its execution, or
	  * false if it must be ended
	  **/
	public boolean activated(Object taskParam);
}
